{-|
Module      : Async.Worker.Broker.PGMQ
Description : PGMQ broker implementation
Copyright   : (c) Gargantext, 2024-Present
License     : AGPL
Maintainer  : gargantext@iscpif.fr
Stability   : experimental
Portability : POSIX
-}

{-# LANGUAGE FlexibleInstances #-}
{-# LANGUAGE MultiParamTypeClasses #-}
{-# LANGUAGE RankNTypes #-}
{-# LANGUAGE ScopedTypeVariables #-}
    
module Async.Worker.Broker.PGMQ
  ( PGMQBroker(..) )
where

import Async.Worker.Broker (Broker(..), Queue)
import Database.PostgreSQL.Simple qualified as PSQL
import Database.PGMQ.Simple qualified as PGMQ
import Database.PGMQ.Types qualified as PGMQ


type ConnParams = PSQL.ConnectInfo
type Timeout = Int

    
data PGMQBroker =
  PGMQBroker {
      conn :: PSQL.Connection
    }

instance (PGMQ.MessageClass m) => Broker PGMQBroker m where
  createQueue (PGMQBroker { conn }) queue = do
    PGMQ.initialize conn
    PGMQ.createQueue conn queue

  readMessageWaiting q@(PGMQBroker { conn }) queue = do
    mMsg <- PGMQ.readMessageWithPoll conn queue 10 5 100
    case mMsg of
      Nothing -> readMessageWaiting q queue
      Just (PGMQ.Message { message }) -> return message

  sendMessage (PGMQBroker { conn }) queue msg =
    PGMQ.sendMessage conn queue msg 0

  -- -- TODO
  -- ackMessage :: (PGMQQueue a) -> QueueName -> (Job a) -> IO ()
  -- ackMessage _ _ _ = return ()

  -- onMessageFetched :: (PGMQQueue a) -> QueueName -> (Job a) -> IO ()
  -- onMessageFetched q@(PGMQQueue { conn }) queueName msg = do
  --   -- Immediately set visibility timeout of a job, when it's
  --   -- specified in job's metadata (stateVisibilityTimeout is a
  --   -- global property of the worker and might not reflect job's
  --   -- timeout specs)
  --   PGMQ.setMessageVt conn queue msgId (getJobTimeout q msg)

  -- handleJob :: (PGMQQueue a) -> (Job a) -> IO ()
  -- handleJob 
