{-|
Module      : Async.Worker.Broker.Types
Description : Typeclass for the underlying broker that powers the worker
Copyright   : (c) Gargantext, 2024-Present
License     : AGPL
Maintainer  : gargantext@iscpif.fr
Stability   : experimental
Portability : POSIX
-}


{-# LANGUAGE AllowAmbiguousTypes #-}
{-# LANGUAGE MultiParamTypeClasses #-}
    
module Async.Worker.Broker.Types
where
    

type Queue = String


{-| A message in the queue system must have some properties. In
  particular, it must have some sort of 'id'.
-}
class BMessage m where
  messageId :: m -> a

    
{-|
  This is an interface for basic broker functionality.
-}
class Broker b m where
  {-| Create new queue with given name. Optionally any other
    initializations can be added here. -}
  createQueue :: b -> Queue -> IO ()

  {-| Drop queue -}
  dropQueue :: b -> Queue -> IO ()

  {-| Read message, waiting for it if not present -}
  readMessageWaiting :: (BMessage m) => b -> Queue -> IO m

  {-| Send message -}
  sendMessage :: (BMessage m) => b -> Queue -> m -> IO ()

  {-| Delete message -}
  deleteMessage :: (BMessage m) => b -> Queue -> m -> IO ()

  {-| Archive message -}
  archiveMessage :: (BMessage m) => b -> Queue -> m -> IO ()

    
  -- NOTE This is worker-specific
    
  -- -- | After message was processed, we want to either requeue it or
  -- --   remove it from queue
  -- ackMessage :: (QMessage m) => b -> Queue -> m -> IO ()

  -- {-| Preliminary code to run after message if fetched from queue -}
  -- onMessageFetched :: (QMessage m) => b -> Queue -> m -> IO ()

  -- {-| A way to handle the computation of a job -}
  -- handleJob :: (QMessage m) => b -> m -> IO ()

